# -*- coding: utf-8 -*-

"""A getter for BiGG."""

from datetime import datetime

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "BiGGGetter",
]

URL = "http://bigg.ucsd.edu/api/v2/database_version"


class BiGGGetter(Getter):
    """A getter for BiGG."""

    name = "BiGG"
    version_type = VersionType.semver

    def get(self):
        """Get the latest BiGG version number."""
        res = requests.get(URL).json()
        date = datetime.fromisoformat(res["last_updated"])
        return dict(
            version=res["bigg_models_version"],
            date=date,
        )


if __name__ == "__main__":
    BiGGGetter.print()



# -*- coding: utf-8 -*-

"""Sources for Bioversions."""

import ftplib
import logging
from functools import lru_cache
from typing import Iterable, List, Mapping, Optional, Tuple, Type, Union

from tqdm import tqdm

from .antibodyregistry import AntibodyRegistryGetter
from .bigg import BiGGGetter
from .biogrid import BioGRIDGetter
from .cellosaurus import CellosaurusGetter
from .chebi import ChEBIGetter
from .chembl import ChEMBLGetter
from .chemidplus import ChemIDplusGetter
from .civic import CiVICGetter
from .complexportal import ComplexPortalGetter
from .daily import NCBIGeneGetter
from .depmap import DepMapGetter
from .dgi import DGIGetter
from .disgenet import DisGeNetGetter
from .drugbank import DrugBankGetter
from .drugcentral import DrugCentralGetter
from .ensembl import EnsemblGetter
from .expasy import ExPASyGetter
from .flybase import FlybaseGetter
from .guidetopharmacology import GuideToPharmacologyGetter
from .hgnc import HGNCGetter
from .homologene import HomoloGeneGetter
from .icd10 import ICD10Getter
from .icd11 import ICD11Getter
from .icf import ICFGetter
from .intact import IntActGetter
from .interpro import InterProGetter
from .itis import ITISGetter
from .kegg import KEGGGetter
from .mesh import MeshGetter
from .mgi import MGIGetter
from .mirbase import MirbaseGetter
from .moalmanac import MOAlmanacGetter
from .msigdb import MSigDBGetter
from .ncit import NCItGetter
from .npass import NPASSGetter
from .obo import iter_obo_getters
from .ols import extend_ols_getters
from .omim import OMIMGetter
from .oncotree import OncoTreeGetter
from .pathbank import PathBankGetter
from .pathwaycommons import PathwayCommonsGetter
from .pfam import PfamGetter
from .pombase import PombaseGetter
from .pr import PRGetter
from .pubchem import PubChemGetter
from .reactome import ReactomeGetter
from .rfam import RfamGetter
from .rgd import RGDGetter
from .rhea import RheaGetter
from .rxnorm import RxNormGetter
from .sgd import SgdGetter
from .slm import SwissLipidGetter
from .stringdb import StringDBGetter
from .umls import UMLSGetter
from .uniprot import UniProtGetter
from .wikipathways import WikiPathwaysGetter
from .zfin import ZfinGetter
from ..utils import Bioversion, Getter, norm, refresh_daily

__all__ = [
    "resolve",
    "get_rows",
    "get_version",
]

logger = logging.getLogger(__name__)


@lru_cache(maxsize=1)
def get_getters() -> List[Type[Getter]]:
    """Get a list of getters."""
    # TODO replace with entrypoint lookup
    getters: List[Type[Getter]] = [
        BioGRIDGetter,
        ChEMBLGetter,
        ComplexPortalGetter,
        DrugBankGetter,
        DrugCentralGetter,
        ExPASyGetter,
        IntActGetter,
        InterProGetter,
        ReactomeGetter,
        RfamGetter,
        WikiPathwaysGetter,
        MirbaseGetter,
        MSigDBGetter,
        PfamGetter,
        UniProtGetter,
        KEGGGetter,
        PathBankGetter,
        NCBIGeneGetter,
        NPASSGetter,
        RheaGetter,
        StringDBGetter,
        HomoloGeneGetter,
        DisGeNetGetter,
        MeshGetter,
        DGIGetter,
        FlybaseGetter,
        PombaseGetter,
        SgdGetter,
        ZfinGetter,
        NCItGetter,
        RxNormGetter,
        ChemIDplusGetter,
        GuideToPharmacologyGetter,
        OncoTreeGetter,
        MOAlmanacGetter,
        AntibodyRegistryGetter,
        EnsemblGetter,
        BiGGGetter,
        ChEBIGetter,
        PRGetter,
        PubChemGetter,
        SwissLipidGetter,
        ITISGetter,
        DepMapGetter,
        PathwayCommonsGetter,
        UMLSGetter,
        HGNCGetter,
        RGDGetter,
        CellosaurusGetter,
        MGIGetter,
        OMIMGetter,
        ICFGetter,
        ICD10Getter,
        ICD11Getter,
        CiVICGetter,
    ]
    getters.extend(iter_obo_getters())
    extend_ols_getters(getters)
    getters = sorted(getters, key=lambda cls: (cls.bioregistry_id or "", cls.__name__.casefold()))
    return getters


def get_getter_dict() -> Mapping[str, Type[Getter]]:
    """Get a dict of getters."""
    rv = {}
    for getter in get_getters():
        if getter.bioregistry_id:
            rv[getter.bioregistry_id] = getter
            rv[norm(getter.bioregistry_id)] = getter
        rv[getter.name] = getter
        rv[norm(getter.name)] = getter
        for pp in getter.collection or []:
            rv[pp] = getter
            rv[norm(pp)] = getter
    return rv


def resolve(name: str, use_cache: bool = True) -> Bioversion:
    """Resolve the database name to a :class:`Bioversion` instance."""
    if use_cache:
        return _resolve_helper_cached(name)
    else:
        return _resolve_helper(name)


@refresh_daily
def _resolve_helper_cached(name: str) -> Bioversion:
    return _resolve_helper(name)


def _resolve_helper(name: str) -> Bioversion:
    norm_name = norm(name)
    getter: Type[Getter] = get_getter_dict()[norm_name]
    return getter.resolve()


def get_version(name: str) -> str:
    """Resolve a database name to its version string."""
    return resolve(name).version


def get_rows(use_tqdm: Optional[bool] = False) -> List[Bioversion]:
    """Get the rows, refreshing once per day."""
    return [
        bioversion
        for bioversion, error in _iter_versions(use_tqdm=use_tqdm)
        if error is None and bioversion is not None
    ]


def _iter_versions(
    use_tqdm: Optional[bool] = False,
) -> Iterable[Union[Tuple[Bioversion, None], Tuple[None, str]]]:
    it = tqdm(get_getters(), disable=not use_tqdm)

    for cls in it:
        it.set_postfix(name=cls.name)
        try:
            yv = resolve(cls.name)
        except (IOError, AttributeError, ftplib.error_perm):
            msg = f"failed to resolve {cls.name}"
            tqdm.write(msg)
            yield None, msg
        except ValueError as e:
            msg = f"issue parsing {cls.name}: {e}"
            tqdm.write(msg)
            yield None, msg
        else:
            yield yv, None



# -*- coding: utf-8 -*-

"""Generation of charts summarizing bioversions."""

import os
from collections import Counter

import click
import matplotlib.pyplot as plt
import seaborn as sns

from bioversions.sources import get_getters
from bioversions.utils import IMG, VersionType

sns.set(style="whitegrid")


def version_types_pie_chart():
    """Make a pie chart with types of versions."""
    counts = Counter(
        "Missing" if getter.version_type is None else getter.version_type.value
        for getter in get_getters()
    )
    labels, counts = zip(*counts.most_common())
    fig, ax = plt.subplots()
    ax.pie(
        counts,
        labels=labels,
        autopct="%1.f%%",
        startangle=30,
        explode=[0.01 for _ in range(len(counts))],
        # shadow=True,
    )
    # fig.legend(fontsize='medium')
    fig.tight_layout()
    path = os.path.join(IMG, "version_types.svg")
    plt.savefig(path, dpi=300)
    plt.close(fig)


def verioning_date_formats_pie_chart():
    """Make a pie chart with types of date/month versions."""
    counts = Counter(
        getter.date_version_fmt
        for getter in get_getters()
        if getter.version_type in {VersionType.date, VersionType.month}
    )
    labels, counts = zip(*counts.most_common())
    fig, ax = plt.subplots()
    ax.pie(
        counts,
        labels=labels,
        autopct="%1.f%%",
        startangle=30,
        explode=[0.01 for _ in range(len(counts))],
    )
    fig.tight_layout()
    path = os.path.join(IMG, "version_date_types.svg")
    plt.savefig(path, dpi=300)
    plt.close(fig)


def has_release_url():
    """Make a pie chart for how many have a release URL."""
    counts = Counter(
        "Has Stable Version URL" if getter.homepage_fmt is not None else "No Stable Version URL"
        for getter in get_getters()
        if getter.version_type != VersionType.unversioned
    )
    labels, counts = zip(*counts.most_common())
    fig, ax = plt.subplots()
    ax.pie(
        counts,
        labels=labels,
        autopct="%1.f%%",
        startangle=30,
        explode=[0.01 for _ in range(len(counts))],
    )
    fig.tight_layout()
    path = os.path.join(IMG, "has_release_url.svg")
    plt.savefig(path, dpi=300)
    plt.close(fig)


@click.command()
def charts():
    """Generate charts for bioversions."""
    version_types_pie_chart()
    verioning_date_formats_pie_chart()
    has_release_url()


if __name__ == "__main__":
    charts()



# -*- coding: utf-8 -*-

"""A getter for the `Drug Gene Interaction Database (DGI-DB) <http://www.dgidb.org>`_."""

import bs4
import dateutil.parser
import requests

from bioversions.utils import Getter, VersionType

GITHUB_PAGE = "https://github.com/dgidb/dgidb-v5"


class DGIGetter(Getter):
    """A getter for DGI."""

    name = "Drug Gene Interaction Database"
    version_type = VersionType.month
    date_version_fmt = "%Y-%b"

    def get(self):
        """Get the latest DGI version number."""
        res = requests.get(GITHUB_PAGE)
        soup = bs4.BeautifulSoup(res.content, features="html.parser")
        time_tag = soup.find("relative-time")
        if time_tag is None:
            raise ValueError
        datetime_str = time_tag.attrs["datetime"]
        dt_obj = dateutil.parser.parse(datetime_str)
        version = dt_obj.strftime(self.date_version_fmt)
        return version


if __name__ == "__main__":
    DGIGetter.print()



# -*- coding: utf-8 -*-

"""Tests for bioversions."""

import datetime
import unittest

import bioregistry

import bioversions
from bioversions.sources import BioGRIDGetter, WikiPathwaysGetter, get_getters


class TestGetter(unittest.TestCase):
    """Tests for the Getter class."""

    def test_bioregistry_ids(self):
        """Test Bioregistry prefixes are all canonical."""
        prefixes = set(bioregistry.read_registry())
        for getter in get_getters():
            if getter.bioregistry_id is None:
                continue
            with self.subTest(name=getter.name):
                self.assertIn(getter.bioregistry_id, prefixes)

    def test_get(self):
        """Test getters."""
        prefixes = [
            "reactome",
            "kegg",
        ]
        for prefix in prefixes:
            with self.subTest(prefix=prefix):
                s = bioversions.get_version(prefix)
                self.assertIsInstance(s, str)

    def test_getter(self):
        """Test the BioGRID getter."""
        s = BioGRIDGetter.version
        self.assertIsInstance(s, str)

    def test_date(self):
        """Test getters that have versions as dates."""
        for getter in [WikiPathwaysGetter]:
            with self.subTest(getter=getter.name):
                s = getter.version
                self.assertIsInstance(s, str)

                d = getter.version_date_parsed
                self.assertIsInstance(d, datetime.date)



# -*- coding: utf-8 -*-

"""A getter for ChEBI."""

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "ChEBIGetter",
]

URL = "https://ftp.ebi.ac.uk/pub/databases/chebi/archive/"


class ChEBIGetter(Getter):
    """A getter for ChEBI."""

    bioregistry_id = "chebi"
    name = "ChEBI"
    version_type = VersionType.sequential
    date_fmt = "%d-%b-%Y"
    homepage_fmt = "https://ftp.ebi.ac.uk/pub/databases/chebi/archive/rel{version}/"

    def get(self):
        """Get the latest ChEBI version number."""
        soup = get_soup(URL)
        anchors = soup.find("pre").find_all("a")
        last = list(anchors)[-1]
        date = last.next_sibling.strip().split()[0]
        version = last.text.rstrip("/")[len("rel") :]
        return dict(version=version, date=date)


if __name__ == "__main__":
    ChEBIGetter.print()



# -*- coding: utf-8 -*-

"""Resources."""

import datetime
import json
from pathlib import Path

import yaml

__all__ = [
    "VERSIONS_PATH",
    "EXPORT_PATH",
    "load_versions",
    "write_versions",
    "write_export",
]

HERE = Path(__file__).parent.resolve()
VERSIONS_PATH = HERE.joinpath(HERE, "versions.json")

ROOT = HERE.parent.parent.parent.resolve()
DOCS = ROOT.joinpath("docs")
EXPORTS_DIRECTORY = DOCS.joinpath("_data")
EXPORT_PATH = EXPORTS_DIRECTORY.joinpath("versions.yml")
FAILURES_PATH = DOCS.joinpath("failures.md")


def load_versions():
    """Load Bioversions data."""
    with open(VERSIONS_PATH) as file:
        return json.load(file)


def _date_converter(o):
    if isinstance(o, (datetime.datetime, datetime.date)):
        return o.strftime("%Y-%m-%d")


def write_versions(versions, indent: int = 2, **kwargs):
    """Write Bioversions data."""
    with open(VERSIONS_PATH, "w") as file:
        json.dump(versions, file, indent=indent, default=_date_converter, **kwargs)


def write_export(versions):
    """Write Bioversions data to the export directory."""
    with open(EXPORT_PATH, "w") as file:
        yaml.safe_dump(versions, file)
